<?php

namespace FiloBlu\Flow\Model\Channel\In;

use Magento\Framework\ObjectManagerInterface;

/**
 * Class AbstractModel
 * @package FiloBlu\Flow\Model\Channel\In
 */
abstract class AbstractModel
{
    /**
     * @var \FiloBlu\Flow\Model\Map $map
     */
    public $map;
    /**
     * @var
     */
    protected $_name;
    /**
     * @var
     */
    protected $_data;
    /**
     * @var \FiloBlu\Flow\Model\Connector\AbstractConnector
     */
    protected $_connector;
    /**
     * @var
     */
    protected $_parser;
    /**
     * @var
     */
    protected $_config;
    /**
     * @var \FiloBlu\Flow\Model\Channel\Config
     */
    protected $_configModel;
    /**
     * @var \FiloBlu\Flow\Model\Channel\ConfigFactory
     */
    protected $channelConfigFactory;
    /**
     * @var
     */
    protected $_file;
    /**
     * @var array
     */
    protected $_insertQueue = [];
    /**
     * @var
     */
    protected $_tablename;
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * AbstractModel constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \FiloBlu\Flow\Model\Channel\ConfigFactory $channelConfigFactory
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\ConfigFactory $channelConfigFactory,
        ObjectManagerInterface $objectManager
    )
    {
        $this->_logger = $loggerProvider->getLogger();
        $this->_configModel = $channelConfigFactory->create();
        $this->objectManager = $objectManager;
        $this->channelConfigFactory = $channelConfigFactory;
    }

    /**
     * @param $channelName
     * @throws \Exception
     */
    public function init($channelName)
    {
        $this->loadConfig($channelName);
    }

    /**
     * @param $name
     * @return bool
     * @throws \Exception
     */
    public function loadConfig($name)
    {

        // $this->_logger->info($name);

        // $name -> DB flow_channels.name
        // unique value (string)

        $this->_configModel->load($name);

        if ($this->_configModel->initialized) {
            $this->_config = $this->_configModel->get('config');
            $this->setConnector($this->_configModel->get('connector'));
            $this->setParser($this->_configModel->get('parser'));
            return true;
        }

        return false;
    }

    /**
     * @param $data
     * @return bool
     */
    public function insertData($data)
    {
        // Genero il FLusso ITEMS
        $model = $this->objectManager->create('FiloBlu\Flow\Model\\' . $this->getConfig('flow'));

        if (!$model) {
            return false;
        }

        $model->setData('meta_file', $this->getFile()->getId());

        foreach ($data as $field => $value) {
            $fieldMap = $this->map->get($field);
            if (!isset($fieldMap['skip'])) {
                $model->setData($fieldMap['to'], $value);
            }
        }

        $model->save();
    }

    /**
     * @param $key
     * @return null
     */
    public function getConfig($key)
    {
        if (isset($this->_config[$key])) {
            return $this->_config[$key];
        }
        return null;
    }

    /**
     * @return mixed
     */
    public function getFile()
    {
        return $this->_file;
    }

    /**
     * @param $file
     * @return $this
     */
    public function setFile($file)
    {
        $this->_file = $file;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @param $name
     */
    public function setName($name)
    {
        $this->_name = $name;
    }

    /**
     * @param $config
     * @return $this
     */
    public function load($config)
    {
        $this->_data = $config;
        return $this;
    }

    /**
     *
     */
    public function storeData()
    {

    }

    /**
     * @return \FiloBlu\Flow\Model\Connector\AbstractConnector
     */
    public function getConnector()
    {
        return $this->_connector;
    }

    /**
     * @param $connector
     * @throws \Exception
     */
    public function setConnector($connector)
    {

        $connectorType = $connector['type'];

        $connectorClass = $this->objectManager->create("FiloBlu\Flow\Model\Connector\\" . ucfirst($connectorType));

        if (trim($connectorType) && ($connectorClass instanceof \FiloBlu\Flow\Model\Connector\AbstractConnector)) {
            $this->_connector = $connectorClass;
            $this->_connector->setConfig($connector);
            return;
        }

        throw new \Exception('Error in connector config');

    }

    /**
     * @return mixed
     */
    public function getParser()
    {
        return $this->_parser;
    }

    /**
     * @param $parser
     * @throws \Exception
     */
    public function setParser($parser)
    {
        $parserType = $parser['type'];

        $parser_class = $this->objectManager->create("FiloBlu\Flow\Model\Parser\\" . ucfirst($parserType));

        if (trim($parserType) and ($parser_class instanceof \FiloBlu\Flow\Model\Parser\AbstractParser)) {
            $this->_parser = $parser_class;
            $this->_parser->setConfig($parser);
            return;
        }

        throw new \Exception('Error in parser config');
    }

    /**
     *
     */
    public function processBeginningOfFlow()
    {

    }

    /**
     *
     */
    public function processEndOfFlow()
    {

    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function rlist()
    {
        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        if ($this->getData('depends_folder')) {
            $depends_resources = $this->_connector->listDir($this->getData('depends_folder'));
            if (count($depends_resources) > 0) {
                return false;
            }
        }

        if ($this->getData('depends_file')) {
            $depends_resources = $this->_connector->getResources();;
            if (count($depends_resources) > 0) {
                return false;
            }
        }

        $resources = $this->_connector->getResources();

        if (!count($resources)) {
            return false;
        }

        return $resources;
    }

    /**
     * @param $key
     * @return null
     */
    public function getData($key)
    {
        if (isset($this->_data[$key])) {
            return $this->_data[$key];
        }

        return null;
    }

    /**
     * @param $file
     * @return mixed
     * @throws \Exception
     */
    public function receiveFile($file)
    {
        if (!$this->_connector) {
            throw new \Exception('No connector joined to channel');
        }

        return $this->_connector->receiveResource($file->getName());
    }

    /**
     * @param $file
     * @return bool
     * @throws \Exception
     */
    public function parse($file)
    {
        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        if (!$this->_parser) {
            throw new \Exception('no parser joined to channel');
        }

        $this->setFile($file);
        $localDir = $this->_connector->getLocalDir();

        $this->_parser->loadMap($file->getChannel());
        $this->setMap($this->_parser->map);
        $this->_parser->setChannel($this);

        //Il Parser chiama il channel con il metodo insertData

        $this->_parser->process($localDir . DIRECTORY_SEPARATOR . $file->getName());

        return true;
    }

    /**
     * @param \FiloBlu\Flow\Model\Map $map
     *
     * @return $this
     */
    public function setMap($map)
    {
        $this->map = $map;
        return $this;
    }

    /**
     *
     */
    public function processendingofFlow()
    {

    }

    /**
     * @return bool
     */
    public function commitData()
    {
        return true;
    }
}
