<?php

namespace FiloBlu\Flow\Model\From\Entity;

/**
 * Class Product
 * @package FiloBlu\Flow\Model\From\Entity
 */
class Product extends AbstractEntity
{

    /**
     * @return bool|void
     * @throws \Exception
     */
    protected function setObjectAttributeValue()
    {

        if (!$this->getObjectId()) {
            $message = sprintf('No id found for model %s with given identifier %s', $this->getEntityModel(), $this->_entityIdentifier);
            $this->logger->info('ITEM SKIPPED: ' . $message);
            return false;
        }

        $attribute_code = $this->getAttribute()->getAttributeCode();

        $pids = [$this->getObjectId()];

        $attributesToCopy = $this->_registry->registry('attribute_configurable_copy');

        $fatherProductsIds = $this->helperProduct->getParentProductsIds($pids);

        if (!empty($fatherProductsIds) && in_array($attribute_code, $attributesToCopy)) {
            $pids = array_merge($pids, $fatherProductsIds);
        }

        $attributeValue = $this->getAttributeValue();

        $attributesData = [$attribute_code => $attributeValue];
        try {
            if ($attribute_code != 'sku') {
                \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')
                    ->updateAttributes($pids, $attributesData, $this->_storeId);
            }
        } catch (\Exception $e) {
            $message = "Problem while processing eavs,<br/> store id : " . $this->_storeId . "<br/> pids : " . print_r($pids, true) . "<br/> attribute data : " . print_r($attributesData, true);
            $message .= "<br/> \Exception : " . $e->getMessage();
            $message .= "<br/> Backtrace : <br/> " . nl2br($e->getTraceAsString());
            $this->logger->info($message);
            throw new \Exception($e->getMessage());
        }

    }

    /**
     * @return bool|mixed|null
     * @throws \Exception
     */
    public function getObjectId()
    {

        if (is_null($this->_objectEntityId)) {
            $this->_objectEntityId = \Magento\Framework\App\ObjectManager::getInstance()->create($this->getEntityModel())->getIdBySku($this->_entityIdentifier);
            if (empty($this->_objectEntityId)) {
                $this->_objectEntityId = false;
            }
        }

        return $this->_objectEntityId;
    }

}
