<?php

namespace FiloBlu\Flow\Model\From;

/**
 * Class Product
 * Used in process of Item (Products)
 * @package FiloBlu\Flow\Model\From
 */
class Product extends AbstractFrom
{
    /**
     * @var string
     */
    const CONFIG_FLOW_WEBSITES_MAPPING = 'filoblu_flow/flow_ite/masterfile_website_mapping';
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    /**
     * @var \Magento\Framework\Model\Context
     */
    protected $_context;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var array
     */
    protected $_attributeSets = [];
    /**
     * @var array
     */
    protected $_taxClasses = [];
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var \FiloBlu\Flow\Helper\Product
     */
    protected $helper_product;
    /**
     * @var \FiloBlu\Flow\Helper\Data
     */
    protected $helper_flow;
    /**
     * @var \FiloBlu\Flow\Helper\Attribute
     */
    protected $helper_attribute;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\Catalog\Model\Product\Action
     */
    protected $_productAction;

    /**
     * Product constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \FiloBlu\Flow\Helper\Product $helper_product
     * @param \FiloBlu\Flow\Helper\Data $helper_flow
     * @param \FiloBlu\Flow\Helper\Attribute $helper_attribute
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory $attributeSetCollectionFactory
     * @param \Magento\Tax\Model\TaxClass\Source\Product $taxClassCollection
     * @param \Magento\Catalog\Model\Product\Action $productAction
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\Attribute $helper_attribute,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory $attributeSetCollectionFactory,
        \Magento\Tax\Model\TaxClass\Source\Product $taxClassCollection,
        \Magento\Catalog\Model\Product\Action $productAction,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    )
    {
        $this->_scopeConfig = $scopeConfig;
        $this->_context = $context;
        $this->helper_product = $helper_product;
        $this->helper_flow = $helper_flow;
        $this->helper_attribute = $helper_attribute;
        $this->storeManager = $storeManager;
        $this->productFactory = $productFactory;
        $this->_productAction = $productAction;
        $this->_logger = $loggerProvider->getLogger();
        $this->getProductAttributeSetCollection($attributeSetCollectionFactory);
        $this->getProductTaxClassCollection($taxClassCollection);
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Loads all product attribute set in an array
     *
     * @param $collection
     */
    public function getProductAttributeSetCollection($collection)
    {
        $attributeSetCollection = $collection->create()->addFieldToFilter('entity_type_id', 4)->toArray();
        foreach ($attributeSetCollection['items'] as $item) {
            $this->_attributeSets[$item['attribute_set_name']] = $item['attribute_set_id'];
        }
    }

    /**
     * Loads all product tax class in an array
     *
     * @param $collection
     */
    public function getProductTaxClassCollection($collection)
    {
        $taxClasses = $collection->getAllOptions();
        foreach ($taxClasses as $taxClass) {
            $label = 'None';
            if (!is_object($taxClass['label'])) {
                $label = $taxClass['label'];
            }
            $this->_taxClasses[$label] = $taxClass['value'];
        }
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Product');
    }

    /**
     * @return $this
     * @throws \Exception
     */
    public function process()
    {
        $url_trail = '';

        $ProdHelper = $this->helper_product;

        //Get Attribute Dinamically
        $attributeSetId = $this->getProductAttributeSetFromName($this->getAttributeSet());
        if (empty($attributeSetId)) {
            $message = sprintf("Product attribute_set not defined or wrong! Aborting Process");
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $sku_configurabile = $this->getCodiceConfigurabile();

        $TaxClassCode = trim($this->getFasciaIvaArticoli());
        //Get Dynamic Class
        $mappedTaxClassId = $this->getProductTaxClassFromName($TaxClassCode);
        if (!$mappedTaxClassId) {
            $message = sprintf("tax_class_code '%s' has no Magento tax class associated. Aborting Process", $TaxClassCode);
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $productSku = $this->helper_flow->removeTextSpaces($this->getCodiceArticolo());

        if (empty($productSku)) {
            return $this->error(false, "'codice_articolo' is empty.");
        }

        $model_product = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product');

        $product_id = $model_product->getIdBySku($productSku);

        if (!$product_id) {

            if (!$ProdHelper->createNewProducts()) {
                $message = sprintf('The product has not been found and the module is configured to not create it');
                $this->error(true, $message);
                $this->_logger->info($message);
                return $this;
            }

            if (strrpos($productSku, '-', -1) !== false) {
                $url_trail = 'u';
            }

            $product = $model_product
                ->setTypeId(\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE)
                ->setAttributeSetId($attributeSetId)
                ->setStatus($ProdHelper->getInitialProductStatus())
                ->setVisibility(\Magento\Catalog\Model\Product\Visibility::VISIBILITY_BOTH)
                ->setStoreId(0);

            //Check if the name need to be copied on Description
            if ($this->helper_product->getCopyNameInDescription()) {
                $model_product->setDescription($this->getNomeArticolo());
            }

            //Check if the name need to be copied on Short Description
            if ($this->helper_product->getCopyNameInShortDescription()) {
                $model_product->setShortDescription($this->getNomeArticolo());
            }

            //Check if Disable Module Shop is enabled
            if ($this->helper_product->isDisableShopModuleEnabled()) {
                $disableShopStatus = 0;
                $hidePriceStatus = 0;
                if ($this->helper_product->getDisableShopStatus()) {
                    $disableShopStatus = 1;
                }
                if ($this->helper_product->getHidePriceStatus()) {
                    $hidePriceStatus = 1;
                }
                $model_product->setDisableShop($disableShopStatus);
                $model_product->setHidePrice($hidePriceStatus);
            }

            $model_product->setSku($productSku)
                ->setUrlKey(strtolower(preg_replace('#[^0-9a-z]+#i', '-', $this->getNomeArticolo() . '-' . $productSku)) . $url_trail)
                ->setPrice(0)
                ->setPriceView(0)
                ->setWeight(0)
                ->setName($this->getNomeArticolo());


            if ($ProdHelper->getAddToWebsites()) {

                $websites = $this->storeManager->getWebsites();

                foreach ($websites as $website) {
                    $websitesIds[] = $website->getId();
                }

                $product->setWebsiteIds($websitesIds);
            } else {
                $current_file_id = $this->getMetaFile();
                $inbound_file = $this->helper_flow->getChannelByInbound($current_file_id);
                $current_channel = $inbound_file['channel'];
                $config_mapping = unserialize($this->_scopeConfig->getValue(self::CONFIG_FLOW_WEBSITES_MAPPING));
                $websitesIds = array();
                foreach ($config_mapping as $row_id => $config_row) {
                    if (trim($config_row['channel']) == trim($current_channel))
                        $websitesIds = $config_row['website_id'];
                }

                $product->setWebsiteIds($websitesIds);
            }

        }

        $productAttributesData = array(
            "product_sku" => $this->getCodiceArticolo(),
            "tax_class_id" => $mappedTaxClassId,
            "parent_sku" => $sku_configurabile
        );


        //Setting configurable attributes on simple (if there are any)
        $configurable_attributes = array_filter(explode(",", $this->getConfigurableAttributes()));
        $configurable_values = array_filter(explode(",", $this->getConfigurableValues()));
        $create_configurable = false;
        if (count($configurable_attributes) > 0 && count($configurable_values) == count($configurable_attributes)) {
            $configurableAttributesData = array_combine($configurable_attributes, $configurable_values);
            foreach ($configurableAttributesData as $code => $label) {
                $optionId = $this->helper_attribute->createOrGetId($code, $label);
                $productAttributesData = array_merge($productAttributesData, array($code => $optionId));
            }
            $create_configurable = true;
        }


        //initial stock configuration
        $stockData = $this->getInitialStockData();

        if (!$product_id) {
            //setting the initial stock config...
            $productAttributesData["stock_data"] = $stockData;
            //saving the product...
            $product->addData($productAttributesData);

            try {

                $product->save();

                /* Preset store id */

                $baseUrlKey = $product->getUrlKey();

                foreach ($this->storeManager->getStores(false) as $store) {
                    $storeId=  $store->getStoreId();
                    $this->_productAction->updateAttributes([$product->getId()], ['url_key' => "{$baseUrlKey}_{$storeId}"] ,$storeId);
                }

            } catch (\Exception $e) {
                $message = $e->getMessage();
                $this->_logger->info($message);
            }

        } else {

            // da implementare

            //if (true or !$this->getVisibilitaSuStore()) {
            //$productAttributesData["status"] = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            //}

            //saving attributes...(ERRORE IN UPDATE PRODOTTO)
            //\Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')->updateAttributes(array($product_id), $productAttributesData, 0);

            //saving stock configuration...
            /*
            $stockItem = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\CatalogInventory\Api\StockRegistryInterface')->getStockItem($product_id,0);

            if ( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE && ( $stockData['backorders'] != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO &&
                    ($stockData['min_qty'] == 0 || $stockItem->getQty() > $stockData['min_qty'])
                ) ) {
                $stockItem->setIsInStock(true);
            } elseif( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE ) {
                if($stockItem->getQty() >= $stockItem->getMinSaleQty()){
                    $stockItem->setIsInStock(true);
                }else{
                    $stockItem->setIsInStock(false);
                }
            }

            $stockItem->addData($stockData)->save();
            */

        }

        $className = "\FiloBlu\Flow\Model\Actions\From" . "\\" . ucfirst(str_replace("_", "", "_add_to_configurable"));

        // Need the product to create the configurable!
        if ($create_configurable && !isset($product)) {
            $product = $this->productFactory->create()->load($product_id);
        }

        if ($create_configurable && isset($product) && $product->getId() && class_exists($className)) {
            $specificSpecialClass = \Magento\Framework\App\ObjectManager::getInstance()->get($className);
            // Handling multiple parents for a single simple
            $configurable_skus = $this->getCodiceConfigurabile();
            $configurable_skus_array = explode(",", $configurable_skus);
            foreach ($configurable_skus_array as $single_configurable_sku) {
                $single_configurable_sku = trim($single_configurable_sku);
                $value = $single_configurable_sku . "#" . implode(",", $configurable_attributes);
                //$result = $specificSpecialClass->process($product, $value, 0, $this->getVisibility());
                $result = $specificSpecialClass->process($product, $value, 0);
            }

        }

        return $this;
    }

    /**
     * Get Attribute Set From Name
     *
     * @param $name
     *
     * @return int|mixed|null
     */
    public function getProductAttributeSetFromName($name)
    {
        $setId = 4;
        if (!empty($name)) {
            if (array_key_exists($name, $this->_attributeSets)) {
                //Get Attribute Set Id
                $setId = $this->_attributeSets[$name];
            } else {
                //Throws error later in return
                $setId = null;
            }
        }
        return $setId;
    }

    /**
     * Get Tax Class Id From Name
     *
     * @param $name
     *
     * @return int|mixed|null
     */
    public function getProductTaxClassFromName($name)
    {
        $taxClassId = 2;
        //If empty name it returns the default value
        if (!empty($name) && $name != 'STD') {
            if (array_key_exists($name, $this->_taxClasses)) {
                //Get Tax Class Id
                $taxClassId = $this->_taxClasses[$name];
            } else {
                //Throws error later in return
                $taxClassId = null;
            }
        }
        return $taxClassId;
    }

    /**
     * @return array
     */
    public function getInitialStockData()
    {
        $stockData = array(
            'qty' => 0,
            'is_in_stock' => 0
        );
        $stockData["use_config_min_qty"] = true;
        return $stockData;
    }

    /**
     * @return int
     */
    public function getInitialProductStatus()
    {
        $h = Mage::helper('fbflow/product');
        $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
        if ($h->useSupplierProductStatus()) {
            if (!$this->getVisibilitaSuStore()) {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
            } else {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            }
        } else {
            $status = $h->getDefaultProductStatus();
        }

        return $status;
    }

    public function sendErrorNotifications($file) {
        return true;
    }

}